local AceLocale = LibStub ("AceLocale-3.0")
local Loc = AceLocale:GetLocale ("Details_ChartViewer")

local _math_floor = math.floor --> lua library local
local _cstr = string.format --> lua library local

--> Create the plugin Object
local ChartViewer = _detalhes:NewPluginObject ("Details_ChartViewer", DETAILSPLUGIN_ALWAYSENABLED)
--> Main Frame
local CVF = ChartViewer.Frame

--> desc
ChartViewer:SetPluginDescription ("View data collected by Details! on simple line charts.")

local plugin_version = "v1.3.1"

local function CreatePluginFrames (data)

	function ChartViewer:OnDetailsEvent (event)

		if (event == "HIDE") then
			
		
		elseif (event == "SHOW") then
			if (not ChartViewerWindowFrame.OptionsButton) then
				local options = ChartViewer:GetFramework():NewButton (ChartViewerWindowFrame, nil, "$parentOptionsButton", "OptionsButton", 86, 16, ChartViewer.OpenOptionsPanel, nil, nil, nil, "Options")
				options:SetTextColor (1, 0.93, 0.74)
				options:SetIcon ([[Interface\Buttons\UI-OptionsButton]], 14, 14, nil, {0, 1, 0, 1}, nil, 3)
				options:SetPoint ("right", ChartViewerWindowFrameAddCaptureScriptButton, "left", 0, 0)
			end
			ChartViewer:RefreshScale()
			
		elseif (event == "PLUGIN_DISABLED") then
			ChartViewer:HideButton()
		
		elseif (event == "PLUGIN_ENABLED") then
			ChartViewer:CanShowOrHideButton()
			
		elseif (event ==  "DETAILS_STARTED") then
			--> craete icon
			ChartViewer:CanShowOrHideButton()
		
		elseif (event ==  "DETAILS_DATA_RESET") then
			ChartViewer:CanShowOrHideButton()
			ChartViewerWindowFrame:Hide()
			
		elseif (event ==  "COMBAT_PLAYER_LEAVE") then
			ChartViewer:CanShowOrHideButton()
		end
		
	end
	
----------> Tabs
	
	--new tab
		ChartViewer.tab_prototype = {name = "New Tab", captures = {}, segment_type = 1}
		
		function ChartViewer:GetTab (tabindex)
			return self.tabs [tabindex]
		end
		
		function ChartViewer:CreateTab (tabname, segment_type)
		
			if (not tabname) then
				return false
			elseif (string.len (tabname) < 2) then
				ChartViewer:Msg ("The name is too short.")
				return false
			end
			
			if (not segment_type) then
				segment_type = 1
			end
			
			local new_tab = table_deepcopy (ChartViewer.tab_prototype)
			new_tab.name = tabname
			new_tab.segment_type = segment_type
			
			tinsert (ChartViewer.tabs, new_tab)
			
			ChartViewer:TabRefresh (#ChartViewer.tabs)
			return new_tab
		end
		
	--open the panel for new tab creation
		function ChartViewer:OpenNewTabPanel()
			ChartViewerWindowFrameAddTab:Show()
		end
		
	--enable or disable chart add button
		function ChartViewer:EnableOrDisableAddChartButton()
			local selected_tab = ChartViewer:TabGetCurrent()
			if (selected_tab.segment_type == 1) then --current segments
				if (#selected_tab.captures >= 5) then
					ChartViewerWindowFrameNewCaptureButton:Disable()
				else
					ChartViewerWindowFrameNewCaptureButton:Enable()
				end
				
			elseif (selected_tab.segment_type == 2) then --last 5 segments
				if (#selected_tab.captures >= 1) then
					ChartViewerWindowFrameNewCaptureButton:Disable()
				else
					ChartViewerWindowFrameNewCaptureButton:Enable()
				end
				
			end
		end
	
	--refresh tabs
		function ChartViewer:TabRefresh (tab_selected)
			
			--put in order
			for index, tab in ipairs (ChartViewer.tabs) do 
				local frame = ChartViewer:TabGetFrame (index)
				frame:Show()
				frame:SetText (tab.name)
				frame.index = index
				frame:SetPoint ("topleft", frame:GetParent(), "topleft", 50 + ( (index-1) * 100), -30)
			end
			
			--hide not used tabs
			ChartViewer:TabHideNotUsedFrames()
			
			--check which tab is selected
			if (not tab_selected) then
				tab_selected = ChartViewer.tabs.last_selected
			end
			if (tab_selected > #ChartViewer.tabs) then
				tab_selected = #ChartViewer.tabs
			end
			
			--click on the selected tab
			ChartViewer:TabClick (ChartViewer.tab_container [tab_selected])
		end
	
	--click on a tab
		function ChartViewer:TabClick (self)
			ChartViewer.tabs.last_selected = self.index
			ChartViewer:EnableOrDisableAddChartButton()
			ChartViewer:TabHighlight (self.index)
			ChartViewer:RefreshCapturesDisplays()
			ChartViewer:RefreshGraphic()
		end
		
	--get the selected tab object
		function ChartViewer:TabGetCurrent()
			return ChartViewer.tabs [ChartViewer.tabs.last_selected]
		end
		
	--adds a capture the the tab
		function ChartViewer:TabAddChart (chart_table)
			local current_tab = ChartViewer:TabGetCurrent()
			tinsert (current_tab.captures, chart_table)
			ChartViewer:EnableOrDisableAddChartButton()
			ChartViewer:RefreshCapturesDisplays()
			return true
		end
		
	--delete a tab
		function ChartViewer:TabErase (index)

			--get the new index
			local new_selected_tab = 1
			if (#ChartViewer.tabs > 1) then
				if (index == ChartViewer.tabs.last_selected) then
					if (index > 1 and #ChartViewer.tabs > index) then
						new_selected_tab = index
					elseif (index == #ChartViewer.tabs) then
						ChartViewer:TabRefresh (index-1)
					else
						ChartViewer:TabRefresh (1)
					end
					
				elseif (index < ChartViewer.tabs.last_selected) then
					new_selected_tab = ChartViewer.tabs.last_selected - 1
				else
					new_selected_tab = ChartViewer.tabs.last_selected
				end
			end
			--do the erase thing
			tremove (ChartViewer.tabs, index)
			
			--check if there is at least 1 tab
			if (#ChartViewer.tabs == 0) then
				ChartViewer:CreateTab ("New Tab", 1)
				new_selected_tab = 1
			end
			
			--refresh
			ChartViewer:TabRefresh (new_selected_tab)
		end
		
	--tab frames
		function ChartViewer:TabGetFrame (index)
			local frame = ChartViewer.tab_container [index]
			if (not frame) then
				frame = ChartViewer:TabCreateFrame (index)
				frame.index = index
				tinsert (ChartViewer.tab_container, frame)
			end
			return frame
		end
		function ChartViewer:TabCreateFrame (index)
			local frame = CreateFrame ("button", "ChartViewerTab" .. index, ChartViewerWindowFrame, "ChartViewerTabFrameTemplate")
			--cleanup default scripts
			frame:SetScript ("OnDoubleClick", nil)
			frame:SetScript ("OnDragStart", nil)
			return frame
		end
		function ChartViewer:TabHideNotUsedFrames()
			for i = #ChartViewer.tabs+1, #ChartViewer.tab_container do
				ChartViewer.tab_container [i]:Hide()
			end
		end
		function ChartViewer:TabHighlight (index)
			for i = 1, #ChartViewer.tab_container do
				local tab_frame = ChartViewer:TabGetFrame (i)
				if (i == index) then
					tab_frame.leftSelectedTexture:SetVertexColor (1, .7, 0, 1)
					tab_frame.middleSelectedTexture:SetVertexColor (1, .7, 0, 1)
					tab_frame.rightSelectedTexture:SetVertexColor (1, .7, 0, 1)
				else
					tab_frame.leftSelectedTexture:SetVertexColor (1, 1, 1, 1)
					tab_frame.middleSelectedTexture:SetVertexColor (1, 1, 1, 1)
					tab_frame.rightSelectedTexture:SetVertexColor (1, 1, 1, 1)
				end
			end

		end
	
----------> Tab Chart Capture
		ChartViewer.chartview_prototype = {capture_name = "", color = {1, 1, 1, 1}}
	
	--new chart capture
		function ChartViewer:CreateViewer (capture_name, color)
			if (not capture_name or not color) then
				return false
			end
			
			local new_viewer = table_deepcopy (ChartViewer.chartview_prototype)
			new_viewer.capture_name = capture_name
			new_viewer.color = color
			
			ChartViewer:TabAddChart (new_viewer)
			ChartViewer:RefreshGraphic()
			
			return new_viewer
		end
		
	--open the new capture creation panel
		function ChartViewer:OpenNewCapturePanel (self)
			ChartViewerWindowFrameAddCapture:Show()
		end
		
	--refresh the bottom charts displays
		function ChartViewer:RefreshCapturesDisplays()
		
			local current_tab = ChartViewer:TabGetCurrent()

			for i = 1, #current_tab.captures do 
				local this_capture = current_tab.captures [i]
				local widget = ChartViewer:GetCaptureDisplay (i)
				widget.text:SetText (this_capture.capture_name)
				widget.index = i
				
				local r, g, b = unpack (this_capture.color)
				widget.texture:SetTexture (r, g, b, 0.3)
				widget:Show()
			end
		
			for i = #current_tab.captures+1, 5 do 
				local widget = ChartViewer:GetCaptureDisplay (i)
				widget:Hide()
			end
		end
		
		function ChartViewer:GetCaptureDisplay (index)
			return ChartViewer.capuresdisplay_container [index]
		end
		
	--remove a capture display
		function ChartViewer:RemoveCaptureDisplay (index)
			local current_tab = ChartViewer:TabGetCurrent()
			tremove (current_tab.captures, index)
			ChartViewer:RefreshCapturesDisplays()
			ChartViewer:EnableOrDisableAddChartButton()
			ChartViewer:RefreshGraphic()
		end
	
----------> Icon show functions
		function ChartViewer:ShowButton()
			if (not ChartViewer.ToolbarButton:IsShown()) then
				ChartViewer:ShowToolbarIcon (ChartViewer.ToolbarButton, "star")
			end
		end
		function ChartViewer:HideButton()
			if (ChartViewer.ToolbarButton:IsShown()) then
				ChartViewer:HideToolbarIcon (ChartViewer.ToolbarButton)
			end
		end
		
		function ChartViewer:RefreshScale()
			local scale = ChartViewer.options.window_scale
			if (ChartViewerWindowFrame) then
				ChartViewerWindowFrame:SetScale (scale)
			end
		end
		
		function ChartViewer:CanShowOrHideButton()
		
			if (not ChartViewer.__enabled) then
				return ChartViewer:HideButton()
			end
		
			if (self.options.show_method == 1) then --> always show
				ChartViewer:ShowButton()
				
			elseif (self.options.show_method == 2) then --> group
				if (IsInGroup() or IsInRaid()) then
					ChartViewer:ShowButton()
				else
					ChartViewer:HideButton()
				end
				
			elseif (self.options.show_method == 3) then --> inside instances
				local _, instanceType = GetInstanceInfo()
				if (instanceType == "raid") then
					ChartViewer:ShowButton()
				else
					ChartViewer:HideButton()
				end
				
			elseif (self.options.show_method == 4) then --> automatic
				local combat = ChartViewer:GetCombat ("current")
				if (combat) then
					local charts = combat.TimeData
					if (charts) then
						for id, chart in pairs (charts) do
							if (chart and chart [1] and chart [2]) then
								ChartViewer:ShowButton()
								return
							end
						end
					end
				end
				ChartViewer:HideButton()
			end
		end
		
		ChartViewer.CanShowOrHideButtonEvents = {
			["GROUP_ROSTER_UPDATE"] = true,
			["PARTY_MEMBERS_CHANGED"] = true,
			["PARTY_CONVERTED_TO_RAID"] = true,
			["ZONE_CHANGED_NEW_AREA"] = true,
			["PLAYER_ENTERING_WORLD"] = true,
		}
		
		function ChartViewer:CreateNewDataFeed (templateID)
			local name = ChartViewer.DataFeedTemplatesByIndex [templateID]
			local script = ChartViewer.DataFeedTemplates [name]
			
			if (script) then
				local result = ChartViewer:TimeDataRegister (name, script, nil, "Chart Viewer", "1.0", [[Interface\ICONS\TEMP]], true)
				if (type (result) == "string") then
					ChartViewer:Msg (result)
				else
					ChartViewer:Msg ("Successfully added.")
				end
			end
		end

----------> Graphic Frame
	
	--change line indicators
	function ChartViewer:ClearIndicators()
		for i = 1, 5 do
			local line = self.line_colors ["text" .. i]
			local texture = self.line_colors ["color" .. i]
			line:SetText ("")
			texture:SetTexture (1, 1, 1, 0)
		end
	end
	
	function ChartViewer:SetIndicator (index, name, r, g, b)
		local line = self.line_colors ["text" .. index]
		local texture = self.line_colors ["color" .. index]
		
		line:SetText (name)
		texture:SetTexture (r, g, b)
	end

	--graphic
		local frame = _G.ChartViewerWindowFrame
		local Graphics = LibStub:GetLibrary("LibGraph-2.0")
		
		local g = Graphics:CreateGraphLine ("ChartViewerGraphFrame", frame, "topleft", "topleft", 6, -67, 724, 318)
		g:SetXAxis (-1,1)
		g:SetYAxis (-1,1)
		g:SetGridSpacing (false, false)
		g:SetGridColor ({0.5,0.5,0.5,0.3})
		g:SetAxisDrawing (false,false)
		g:SetAxisColor({1.0,1.0,1.0,1.0})
		g:SetAutoScale (true)
		g:SetLineTexture ("smallline")
		g:SetBorderSize ("right", 0.001)
		g.VerticalLines = {}
		g.TryIndicator = {}
		
		g:SetBackdrop({
				edgeFile = "Interface\\DialogFrame\\UI-DialogBox-gold-Border", tile = true, tileSize = 16, edgeSize = 5,
				insets = {left = 1, right = 1, top = 0, bottom = 1},})
		
		function g:ChangeColorOnDataSeries (index, color)
			self.Data [index].Color = color
			self.NeedsUpdate=true
		end
		
		function g:AddDataSeriesOnFirstIndex (points, color, n2)
			local data
			--Make sure there is data points
			if not points then
				return
			end

			data=points
			if n2==nil then
				n2=false
			end
			if n2 or (table.getn(points)==2 and table.getn(points[1])~=2) then
				data={}
				for k,v in ipairs(points[1]) do
					tinsert(data,{v,points[2][k]})
				end
			end
			
			table.insert (self.Data, 1, {Points=data;Color=color})
			
			self.NeedsUpdate=true
		end

	--timeline
		local time_label = frame:CreateFontString (nil, "overlay", "GameFontHighlightSmall")
		frame.timeamt0 = time_label
		frame.timeamt0:SetText ("00:00")
		frame.timeamt0:SetPoint ("TOPLEFT", frame, "TOPLEFT", 75, -370)
	
	--amount grid
		for i = 1, 8, 1 do
			local line = g:CreateTexture (nil, "overlay")
			line:SetTexture (.5, .5, .5, .7)
			line:SetWidth (710)
			line:SetHeight (1)
			line:SetVertexColor (.4, .4, .4, .8)
		
			local amount = frame:CreateFontString (nil, "overlay", "GameFontHighlightSmall")
			amount:SetText ("100k")
			amount:SetPoint ("TOPLEFT", frame, "TOPLEFT", 23, -61 + (-(31.8*i)))
			frame ["amount_label_" .. i] = amount
			line:SetPoint ("topleft", amount, "bottom", -27, 0)
			
			local timeline = frame:CreateFontString (nil, "overlay", "GameFontHighlightSmall")
			timeline:SetText ("0"..i..":00")
			timeline:SetPoint ("TOPLEFT", frame, "TOPLEFT", 75+(73*i), -370)
			frame ["timeamt"..i] = timeline
		end
		
		frame.MaxGraphics = 5
		
		local v = g:CreateTexture (nil, "overlay")
		v:SetWidth (1)
		v:SetHeight (318)
		v:SetPoint ("top", g, "top", 0, 1)
		v:SetPoint ("left", g, "left", 55, 0)
		v:SetTexture (1, 1, 1, 1)
		
		local h = g:CreateTexture (nil, "overlay")
		h:SetWidth (722)
		h:SetHeight (2)
		h:SetPoint ("top", g, "top", 1, -297)
		h:SetPoint ("left", g, "left", 1, 0)
		h:SetTexture (1, 1, 1, 1)
		
		g:ResetData()

	--refresh the graphic
	
		local colors = {{1, 0, 0}, {1, 0.7, 0}, {1, 1, 0}, {0.7, 1, 0}, {0, 1, 0}}
	
		function ChartViewer:RefreshGraphic()
		
			local current_tab = ChartViewer:TabGetCurrent()
			
			local captures = current_tab.captures
			local segment_type = current_tab.segment_type
			
			g:ResetData()
			ChartViewer:ClearIndicators()
			
			local data = {}
			if (segment_type == 1) then --only current
				local combat = ChartViewer:GetCombat ("current")
				local elapsed_time = combat:GetCombatTime()
				if (elapsed_time > 12) then
					for index, chart in ipairs (captures) do 
						local chart_data = combat:GetTimeData (chart.capture_name)
						if (chart_data and chart_data.max_value and chart_data.max_value > 0) then
							tinsert (data, {chart_data, chart.color, elapsed_time})
							ChartViewer:SetIndicator (#data, chart.capture_name, unpack (chart.color))
						end
					end
				end

			elseif (segment_type == 2) then --last 5 segments
			
				local chart = captures [1]
				if (chart) then
					for i = 5, 1, -1 do
						local combat = ChartViewer:GetCombat (i)
						if (combat) then
							local elapsed_time = combat:GetCombatTime()
							if (elapsed_time > 12) then
								local chart_data = combat:GetTimeData (chart.capture_name)
								if (chart_data and chart_data.max_value and chart_data.max_value > 0) then

									if (i == 1) then
										tinsert (data, {chart_data, chart.color, elapsed_time})
										ChartViewer:SetIndicator (#data, chart.capture_name, unpack (chart.color))
									else
										tinsert (data, {chart_data, colors [i], elapsed_time})
										ChartViewer:SetIndicator (#data, "#" .. i, unpack (colors [i]))
									end
								end
							end
						end
					end
				end
			end
			
			if (#data > 0) then
			
				local scale = 1/724
				local maximum_value = 0
				local maximum_time = 0
				
				for index, chart in ipairs (data) do 
				
					--> get the tables and color
					local chart_data = chart [1]
					local chart_color = chart [2]
					local combat_time = chart [3]
					
					--> get the max indexes and max value
					local amount_pieces = #chart_data
					local amount_max = chart_data.max_value
					
					--> normalize the start and end
					tinsert (chart_data, 1, 0)
					tinsert (chart_data, 1, 0)
					tinsert (chart_data, #chart_data+1, 0)
					tinsert (chart_data, #chart_data+1, 0)

					--> update the maximum value
					local max_value = math.max (maximum_value, amount_max)
					
					--> create the normalized graphic table
					local normalized_data = {}
					local i = 3
					while (i <= #chart_data-2) do 
						local v = (chart_data [i-2] + chart_data [i-1] + chart_data [i] + chart_data [i+1] + chart_data [i+2]) / 5
						normalized_data [#normalized_data+1] = {scale*(i-2), v/max_value}
						i = i + 1
					end
			
					--> remove start and end normalization from source table
					tremove (chart_data, 1)
					tremove (chart_data, 1)
					tremove (chart_data, #chart_data)
					tremove (chart_data, #chart_data)
					
					--> update the time line
					if (maximum_time < combat_time) then 
						maximum_time = combat_time
						local time_break = maximum_time / 8
						
						for i = 1, 8 do
							local t = time_break * i
							local minutes, seconds = math.floor (t/60), math.floor (t%60)
							if (seconds < 10) then
								seconds = "0" .. seconds
							end
							if (minutes < 10) then
								minutes = "0" .. minutes
							end
							frame ["timeamt"..i]:SetText (minutes .. ":" .. seconds)
						end
					end
					
					--> update the amount strings
					if (amount_max > maximum_value) then 
					
						--> normalize previous data
						if (maximum_value > 0) then
							local normalizePercent = maximum_value / amount_max
							for data_index, points_data in ipairs (g.Data) do 
								local points = points_data.Points
								for i = 1, #points do 
									points[i][2] = points[i][2] * normalizePercent
								end
							end
						end
					
						maximum_value = amount_max
						
						local damage_break = maximum_value / 8
						local o = 1
						for i = 8, 1, -1 do
							local d = ChartViewer:ToK (damage_break * i)
							frame ["amount_label_" .. o]:SetText (d)
							o = o + 1
						end
						
					end	

					g:AddDataSeriesOnFirstIndex (normalized_data, chart_color)
					--g:AddDataSeries (normalized_data, chart_color)
			
				end
			end
			
		end

----------> Window Functions

	--open window
		function ChartViewer:OpenWindow()
			ChartViewer:TabRefresh()
			ChartViewerWindowFrame:Show()
		end
		
	
----------> Creathe the icon
		ChartViewer.ToolbarButton = _detalhes.ToolBar:NewPluginToolbarButton (ChartViewer.OpenWindow, [[Interface\Addons\Details_ChartViewer\icon]], Loc ["STRING_PLUGIN_NAME"], Loc ["STRING_TOOLTIP"], 14, 14, "CHARTVIEWER_BUTTON")
	
end

function ChartViewer:SetCaptureDropdownFunctions()
	
	--> data feed dropdown
		local datafeed_dropdown = ChartViewerWindowFrameAddDataFeed.datafeed_dropdown

		local function OnClick_datafeed (self)
			UIDropDownMenu_SetSelectedID (datafeed_dropdown, self:GetID())
		end
	
		local feed_list = {
			{"Raid Damage Taken", 1},
			{"Raid Healing Done", 2},
			{"Raid Overheal", 3},
		}
		
		local function initialize_datafeed (self, level)
			for index, value in ipairs (feed_list) do
				local info = UIDropDownMenu_CreateInfo()
				info.text = value[1]
				info.value = value[2]
				info.func = OnClick_datafeed
				UIDropDownMenu_AddButton (info, level)
			end
		end
		
		UIDropDownMenu_Initialize (datafeed_dropdown, initialize_datafeed)
		UIDropDownMenu_SetWidth (datafeed_dropdown, 120)
		UIDropDownMenu_SetButtonWidth (datafeed_dropdown, 120)
		--UIDropDownMenu_SetSelectedValue (datafeed_dropdown, 1)
		UIDropDownMenu_JustifyText (datafeed_dropdown, "LEFT")
	
	--> segments dropdown
		local segment_dropdown = ChartViewerWindowFrameAddTabSegmentTypeDropDown
		
		local function OnClick_segments (self)
			UIDropDownMenu_SetSelectedID (segment_dropdown, self:GetID())
		end
		
		local segment_list = {
			{"Current Segment", 1}, 
			{"Last 5 Segments", 2}
		}
		
		local function initialize_segments (self, level)
			for index, value in ipairs (segment_list) do
				local info = UIDropDownMenu_CreateInfo()
				info.text = value[1]
				info.value = value[2]
				info.func = OnClick_segments
				UIDropDownMenu_AddButton (info, level)
			end
		end
		
		UIDropDownMenu_Initialize (segment_dropdown, initialize_segments)
		UIDropDownMenu_SetWidth (segment_dropdown, 120)
		UIDropDownMenu_SetButtonWidth (segment_dropdown, 120)
		UIDropDownMenu_SetSelectedValue (segment_dropdown, 1)
		UIDropDownMenu_JustifyText (segment_dropdown, "LEFT")
	
	--> chart selection dropdown
		local chart_dropdown = ChartViewerWindowFrameAddCaptureChartDropDown
		
		local function OnClick_charts (self)
			UIDropDownMenu_SetSelectedID (chart_dropdown, self:GetID())
		end
		
		local function initialize_charts (self, level)
			for index, capture in ipairs (_detalhes.savedTimeCaptures) do
				local info = UIDropDownMenu_CreateInfo()
				info.text = capture[1]
				info.value = capture[1]
				info.func = OnClick_charts
				UIDropDownMenu_AddButton (info, level)
			end
		end
		
		UIDropDownMenu_Initialize (chart_dropdown, initialize_charts)
		UIDropDownMenu_SetWidth (chart_dropdown, 120)
		UIDropDownMenu_SetButtonWidth (chart_dropdown, 120)
		UIDropDownMenu_SetSelectedValue (chart_dropdown, 1, true)
		UIDropDownMenu_JustifyText (chart_dropdown, "LEFT")
end

local build_options_panel = function()

	local options_frame = ChartViewer:CreatePluginOptionsFrame ("ChartViewerOptionsWindow", "Chart Viewer Options", 2)
	
	local set = function (_, _, value) 
		ChartViewer.options.show_method = value 
		ChartViewer:CanShowOrHideButton()
	end
	local on_show_menu = {
		{value = 1, label = "Always", onclick = set, desc = "Always show the icon."},
		{value = 2, label = "In Group", onclick = set, desc = "Only show the icon while in group."},
		{value = 3, label = "Inside Raid", onclick = set, desc = "Only show the icon while inside a raid."},
		{value = 4, label = "Auto", onclick = set, desc = "The plugin decides when the icon needs to be shown."},
	}
	
	local menu = {
		{
			type = "select",
			get = function() return ChartViewer.options.show_method end,
			values = function() return on_show_menu end,
			desc = "When the icon is shown over Details! toolbar.",
			name = "Show Icon"
		},
		{
			type = "range",
			get = function() return ChartViewer.options.window_scale end,
			set = function (self, fixedparam, value) ChartViewer.options.window_scale = value; ChartViewer:RefreshScale() end,
			min = 0.65,
			max = 1.50,
			step = 0.1,
			desc = "Set the window size",
			name = "Window Scale",
			usedecimals = true,
		},
	}
	ChartViewer:GetFramework():BuildMenu (options_frame, menu, 15, -75, 260)
end

ChartViewer.OpenOptionsPanel = function()
	if (not ChartViewerOptionsWindow) then
		build_options_panel()
	end
	ChartViewerOptionsWindow:Show()
end

function ChartViewer:OnEvent (_, event, ...)


	if (event == "ADDON_LOADED") then
		local AddonName = select (1, ...)
		if (AddonName == "Details_ChartViewer") then
			
			if (_G._detalhes) then

				--> register player damage done chart data
				ChartViewer:TimeDataRegister ("Player Damage Done", ChartViewer.PlayerDamageDoneChartCode, nil, "Chart Viewer", "1.0", [[Interface\ICONS\Ability_MeleeDamage]], true)
				
				--> create widgets
				CreatePluginFrames()

				local MINIMAL_DETAILS_VERSION_REQUIRED = 17
				
				--> Install
				local install, saveddata, is_enabled = _G._detalhes:InstallPlugin ("TOOLBAR", Loc ["STRING_PLUGIN_NAME"], [[Interface\Addons\Details_ChartViewer\icon]], ChartViewer, "DETAILS_PLUGIN_CHART_VIEWER", MINIMAL_DETAILS_VERSION_REQUIRED, "Details! Team", plugin_version)
				if (type (install) == "table" and install.error) then
					print (Loc ["STRING_PLUGIN_NAME"], install.errortext)
					return
				end

				--> build tab container
				saveddata.tabs = saveddata.tabs or { {segment_type = 2, name = "Your Damage", captures = {{capture_name = "Player Damage Done", color = {1, 1, 1}}}}, last_selected = 1}
				saveddata.options = saveddata.options or {show_method = 4, window_scale = 1.0}
				
				ChartViewer.tabs = saveddata.tabs
				ChartViewer.options = saveddata.options
				
				ChartViewer.tabs.last_selected = ChartViewer.tabs.last_selected or 1
				
				ChartViewer.tab_container = {}
				
				--> register wow events
				CVF:RegisterEvent ("GROUP_ROSTER_UPDATE")
				CVF:RegisterEvent ("PARTY_MEMBERS_CHANGED")
				CVF:RegisterEvent ("PARTY_CONVERTED_TO_RAID")
				CVF:RegisterEvent ("ZONE_CHANGED_NEW_AREA")
				CVF:RegisterEvent ("PLAYER_ENTERING_WORLD")
				
				if (is_enabled) then
					ChartViewer:CanShowOrHideButton()
				else
					ChartViewer:HideButton()
				end
				
				ChartViewer:SetCaptureDropdownFunctions()
				ChartViewer.capuresdisplay_container = {ChartViewerWindowFrameCapture1, ChartViewerWindowFrameCapture2 , ChartViewerWindowFrameCapture3 , ChartViewerWindowFrameCapture4 , ChartViewerWindowFrameCapture5}
				
				local f = ChartViewerWindowFrame
				ChartViewer.line_colors = {
					color1 = f.line1colortexture, text1 = f.line1colortext,
					color2 = f.line2colortexture, text2 = f.line2colortext,
					color3 = f.line3colortexture, text3 = f.line3colortext,
					color4 = f.line4colortexture, text4 = f.line4colortext,
					color5 = f.line5colortexture, text5 = f.line5colortext,
				}
				
				--> register details events
				_G._detalhes:RegisterEvent (ChartViewer, "DETAILS_DATA_RESET")
				_G._detalhes:RegisterEvent (ChartViewer, "COMBAT_PLAYER_LEAVE")

			end
		end

	elseif (ChartViewer.CanShowOrHideButtonEvents [event]) then
		ChartViewer:CanShowOrHideButton()		
		
	end
end

--> player damage done chart code
ChartViewer.PlayerDamageDoneChartCode = [[

	-- the goal of this script is get the current combat then get your character and extract your damage done.
	-- the first thing to do is get the combat, so, we use here the command "_detalhes:GetCombat ( "overall" "current" or "segment number")"
	
	local current_combat = _detalhes:GetCombat ("current") --> getting the current combat
	
	-- the next step is request your character from the combat
	-- to do this, we take the combat which here we named "current_combat" and tells what we want inside parentheses.
	
	local my_self = current_combat (DETAILS_ATTRIBUTE_DAMAGE, _detalhes.playername)
	
	-- _detalhes.playername holds the name of your character.
	-- DETAILS_ATTRIBUTE_DAMAGE means we want the damage table, _HEAL _ENERGY _MISC is the other 3 tables.
	
	-- before we proceed, the result needs to be checked to make sure its a valid result.
	
	if (not my_self) then
		return 0 -- the combat doesnt have *you*, this happens when you didn't deal any damage in the combat yet.
	end
	
	-- now its time to get the total damage.
	
	local my_damage = my_self.total
	
	-- then finally return the amount to the capture.
	
	return my_damage
	
]]

ChartViewer.DataFeedTemplatesByIndex = {
	"Raid Damage Taken", "Raid Healing Done", "Raid Overheal"
}

ChartViewer.DataFeedTemplates = {
["Raid Damage Taken"] = [[ -- this script takes the current combat and get the total of damage taken by the group.
local damage_taken = 0

--get the raid players on current combat
local players = _detalhes:GetActorsOnDamageCache()

--add the damage taken from each player
for _, player in ipairs (players) do
	damage_taken = damage_taken + player.damage_taken
end

--return the value
return damage_taken
]],

["Raid Healing Done"] = [[ -- this script takes the current combat and request the total of healing done by the group.
-- get the current combat
local current_combat = _detalhes:GetCombat ("current")

-- get the total healing done from the combat
local total_healing = current_combat:GetTotal ( DETAILS_ATTRIBUTE_HEAL, nil, DETAILS_TOTALS_ONLYGROUP )

-- check if the result is valid
if (not total_healing) then
	return 0
else
	-- return the value
	return total_healing
end
]],

["Raid Overheal"] = [[ -- this script get the total of overheal from the raid.
local overheal = 0

--get the raid players on current combat
local players = _detalhes:GetActorsOnHealingCache()

--add the overheal from each player
for _, player in ipairs (players) do
	overheal = overheal + player.totalover
end

--return the value
return overheal
]]
}